@@ -17,22 +17,18 @@ test_expect_success 'setup helper scripts' '
 	IFS=$OIFS
 	EOF
 
-	cat >git-credential-useless <<-\EOF &&
-	#!/bin/sh
+	write_script git-credential-useless <<-\EOF &&
 	. ./dump
 	exit 0
 	EOF
-	chmod +x git-credential-useless &&
 
-	cat >git-credential-verbatim <<-\EOF &&
-	#!/bin/sh
+	write_script git-credential-verbatim <<-\EOF &&
 	user=$1; shift
 	pass=$1; shift
 	. ./dump
 	test -z "$user" || echo username=$user
 	test -z "$pass" || echo password=$pass
 	EOF
-	chmod +x git-credential-verbatim &&
 
 	PATH="$PWD:$PATH"
 '
